/* Import data */
proc import out=nishimura
   datafile='C:\chapter10\nishimura CCP.csv'
   dbms=csv
	replace; 
   getnames=yes;
run;

data nishimura_accp; 
	set  nishimura;
  	where test='Anti-CCP';
run;


/* Create a two separate records for the true results in each study, 
   the first for the diseased group, and the second for the non-deiseaed group.
   The variable sens is an indicator which takes the value 1 if true=true positives and 0 otherwise,
   the variable spec is also an indicator that takes the value 1 if true =true negatives and 0 otherwise */
data nishimura_accp; 
	set  nishimura_accp;
	sens=1; spec=0; true=tp; n=tp+fn; output;
	sens=0; spec=1; true=tn; n=tn+fp; output;
run;


/* Ensure that both records for a study are clustered together */
proc sort data=nishimura_accp;
by study_id ;
run;

/* Run the Bivariate model with no covariates
   The "cov" option requests that a covariance matrix is printed for 
   all model parameter estimates. The "ecov" option requests a covariance matrix
   for all additional estimates that are computed. */

proc nlmixed data=nishimura_accp cov ecov ;

/* specify starting values for all parameters to be estimated
   and ensure that the variances of the random effects cannot be negative */

parms msens=1 to 2 by 0.5 mspec=2 to 4 by 0.5 s2usens=0.2 s2uspec=0.6 covsesp=0;
	bounds s2usens>=0;
	bounds s2uspec>=0;
	logitp = (msens + usens)*sens + (mspec + uspec)*spec;
    p = exp(logitp)/(1+exp(logitp)); 
    model true ~ binomial(n,p);

   /* usens and uspec represent the random effects. The are both assumed to be
      normally distributed with mean zero. Their variances estimates are s2usens and s2uspec,
      and their covariance estimate is  covesp */

	random usens uspec ~ normal([0 , 0],[s2usens,covsesp,s2uspec])
			subject=study_id out=randeffs;

/* Additional estimates that are functions of the model parameters can be estimted here:
	e.g the positive and negative likelihood ratios */
estimate 'logLR+' log((exp(msens)/(1+exp(msens)))/(1-(exp(mspec)/(1+exp(mspec)))));
estimate 'logLR-' log((1-(exp(msens)/(1+exp(msens))))/(exp(mspec)/(1+exp(mspec))));

run;


/* Check assumption of normality for the random effects */
proc univariate data=randeffs plot normal;
  class effect;
  var estimate;
run; 

/* Create a dummy variable for CCP generation, coded as 0 for 'CCP1' (the referent generation)
   and coded as 1 for 'CCP2'. This new variable is added to the dataset set created above. */

data nishimura_accp; 
	set nishimura_accp;
	ccpg=0;
	if ccp_generation ="CCP2"  then ccpg=1; 
run;


/* add the covaraite CCPG to the model to allow both sensitivity and specifcity to be
   associated with generation of the test */

proc nlmixed data=nishimura_accp cov ecov;
parms msens=1 mspec=2 s2usens=0.2 s2uspec=0.6 covsesp=0 se2=0 sp2=0;
   bounds s2usens>=0;
   bounds s2uspec>=0;
   logitp=(msens+usens+se2*ccpg)*sens+(mspec+uspec+sp2*ccpg)*spec;
   p = exp(logitp)/(1+exp(logitp)); 
   model true ~ binomial(n,p);
   random usens uspec ~ normal([0 , 0],  [s2usens,covsesp,s2uspec]) 
   subject=study_id out=randeffs;

/* Estimate logit(sensitivity) and logit(specificity) for CCP2 
   (their correlation will be output because of the "ecov" option for nlmixed), 
   and also log likelihood ratios for CCP1 and CCP2 */

estimate 'logitsens CCP2'  msens + se2;
estimate 'logitspec CCP2'  mspec + sp2;
estimate 'logLR+ CCP1' log((exp(msens)/(1+exp(msens)))/(1-(exp(mspec)/(1+exp(mspec)))));
estimate 'logLR- CCP1' log((1-(exp(msens)/(1+exp(msens))))/(exp(mspec)/(1+exp(mspec))));
estimate 'logLR+ CCP2' log((exp(msens+se2)/(1+exp(msens+se2)))/(1-(exp(mspec+sp2)/(1+exp(mspec+sp2)))));
estimate 'logLR- CCP2' log((1-(exp(msens+se2)/(1+exp(msens+se2))))/(exp(mspec+sp2)/(1+exp(mspec+sp2))));
run;


/* Check assumption of normality for the random effects */
proc univariate data=randeffs plot normal;
  class effect;
  var estimate;
run;


